@@ -21,7 +21,7 @@ from pycloudlib import (
     Openstack,
     Qemu,
 )
-from pycloudlib.cloud import BaseCloud, ImageType
+from pycloudlib.cloud import ImageType
 from pycloudlib.ec2.instance import EC2Instance
 from pycloudlib.lxd.cloud import _BaseLXD
 from pycloudlib.lxd.instance import BaseInstance, LXDInstance
@@ -55,7 +55,6 @@ def _get_ubuntu_series() -> list:
 
 class IntegrationCloud(ABC):
     datasource: str
-    cloud_instance: BaseCloud
 
     def __init__(
         self,
@@ -64,7 +63,7 @@ class IntegrationCloud(ABC):
     ):
         self._image_type = image_type
         self.settings = settings
-        self.cloud_instance: BaseCloud = self._get_cloud_instance()
+        self.cloud_instance = self._get_cloud_instance()
         self.initial_image_id = self._get_initial_image()
         self.snapshot_id = None
 
@@ -183,7 +182,7 @@ class IntegrationCloud(ABC):
 
     def delete_snapshot(self):
         if self.snapshot_id:
-            if self.settings.KEEP_IMAGE:
+            if self.settings.KEEP_IMAGE:  # type: ignore
                 log.info(
                     "NOT deleting snapshot image created for this testrun "
                     "because KEEP_IMAGE is True: %s",
@@ -200,7 +199,7 @@ class IntegrationCloud(ABC):
 class Ec2Cloud(IntegrationCloud):
     datasource = "ec2"
 
-    def _get_cloud_instance(self):
+    def _get_cloud_instance(self) -> EC2:
         return EC2(tag="ec2-integration-test")
 
     def _get_initial_image(self, **kwargs) -> str:
@@ -218,17 +217,9 @@ class Ec2Cloud(IntegrationCloud):
                     name="ec2-cloud-init-integration"
                 )
 
-            # Enable IPv6 metadata at http://[fd00:ec2::254]
-            if "Ipv6AddressCount" not in launch_kwargs:
-                launch_kwargs["Ipv6AddressCount"] = 1
-            if "MetadataOptions" not in launch_kwargs:
-                launch_kwargs["MetadataOptions"] = {}
-            if "HttpProtocolIpv6" not in launch_kwargs["MetadataOptions"]:
-                launch_kwargs["MetadataOptions"] = {
-                    "HttpProtocolIpv6": "enabled"
-                }
-
-        pycloudlib_instance = self.cloud_instance.launch(**launch_kwargs)
+        pycloudlib_instance = self.cloud_instance.launch(
+            enable_ipv6=enable_ipv6, **launch_kwargs
+        )
         self._maybe_wait(pycloudlib_instance, wait)
         return pycloudlib_instance
 
@@ -236,7 +227,7 @@ class Ec2Cloud(IntegrationCloud):
 class GceCloud(IntegrationCloud):
     datasource = "gce"
 
-    def _get_cloud_instance(self):
+    def _get_cloud_instance(self) -> GCE:
         return GCE(
             tag="gce-integration-test",
         )
@@ -251,7 +242,7 @@ class AzureCloud(IntegrationCloud):
     datasource = "azure"
     cloud_instance: Azure
 
-    def _get_cloud_instance(self):
+    def _get_cloud_instance(self) -> Azure:
         return Azure(tag="azure-integration-test")
 
     def _get_initial_image(self, **kwargs) -> str:
@@ -273,7 +264,7 @@ class AzureCloud(IntegrationCloud):
 class OciCloud(IntegrationCloud):
     datasource = "oci"
 
-    def _get_cloud_instance(self):
+    def _get_cloud_instance(self) -> OCI:
         return OCI(
             tag="oci-integration-test",
         )
@@ -284,11 +275,7 @@ class _LxdIntegrationCloud(IntegrationCloud):
     instance_tag: str
     cloud_instance: _BaseLXD
 
-    def _get_cloud_instance(self):
-        return self.pycloudlib_instance_cls(tag=self.instance_tag)
-
-    @staticmethod
-    def _get_or_set_profile_list(release):
+    def _get_or_set_profile_list(self, release):
         return None
 
     @staticmethod
@@ -300,6 +287,10 @@ class _LxdIntegrationCloud(IntegrationCloud):
                 os.path.join(cloudinit_path, "..", "templates"),
                 "/etc/cloud/templates",
             ),
+            (
+                os.path.join(cloudinit_path, "..", "doc", "module-docs"),
+                "/usr/share/doc/cloud-init/module-docs",
+            ),
         ]
         for n, (source_path, target_path) in enumerate(mounts):
             format_variables = {
@@ -359,15 +350,21 @@ class LxdContainerCloud(_LxdIntegrationCloud):
     pycloudlib_instance_cls = LXDContainer
     instance_tag = "lxd-container-integration-test"
 
+    def _get_cloud_instance(self) -> LXDContainer:
+        return self.pycloudlib_instance_cls(tag=self.instance_tag)
+
 
 class LxdVmCloud(_LxdIntegrationCloud):
     datasource = "lxd_vm"
     cloud_instance: LXDVirtualMachine
     pycloudlib_instance_cls = LXDVirtualMachine
     instance_tag = "lxd-vm-integration-test"
-    _profile_list = None
+    _profile_list: list = []
 
-    def _get_or_set_profile_list(self, release):
+    def _get_cloud_instance(self) -> LXDVirtualMachine:
+        return self.pycloudlib_instance_cls(tag=self.instance_tag)
+
+    def _get_or_set_profile_list(self, release) -> list:
         if self._profile_list:
             return self._profile_list
         self._profile_list = self.cloud_instance.build_necessary_profiles(
