@@ -2,8 +2,10 @@
 
 import contextlib
 import io
+import json
 import logging
 import os
+import sys
 from collections import namedtuple
 
 import pytest
@@ -15,14 +17,29 @@ from tests.unittests import helpers as test_helpers
 mock = test_helpers.mock
 
 M_PATH = "cloudinit.cmd.main."
+Tmpdir = namedtuple("Tmpdir", ["tmpdir", "link_d", "data_d"])
+FakeArgs = namedtuple("FakeArgs", ["action", "local", "mode"])
 
 
-@pytest.fixture(autouse=False)
-def mock_get_user_data_file(mocker, tmpdir):
-    yield mocker.patch(
-        "cloudinit.cmd.devel.logs._get_user_data_file",
-        return_value=tmpdir.join("cloud"),
-    )
+@pytest.fixture(autouse=True, scope="module")
+def disable_setup_logging():
+    # setup_basic_logging can change the logging level to WARNING, so
+    # ensure it is always mocked
+    with mock.patch(f"{M_PATH}log.setup_basic_logging", autospec=True):
+        yield
+
+
+@pytest.fixture()
+def mock_status_wrapper(mocker, tmpdir):
+    link_d = os.path.join(tmpdir, "link")
+    data_d = os.path.join(tmpdir, "data")
+    with mocker.patch(
+        "cloudinit.cmd.main.read_cfg_paths",
+        return_value=mock.Mock(get_cpath=lambda _: data_d),
+    ), mocker.patch(
+        "cloudinit.cmd.main.os.path.normpath", return_value=link_d
+    ):
+        yield Tmpdir(tmpdir, link_d, data_d)
 
 
 class TestCLI:
@@ -51,43 +68,40 @@ class TestCLI:
             ),
         ],
     )
-    def test_status_wrapper_errors(self, action, name, match, caplog, tmpdir):
-        data_d = tmpdir.join("data")
-        link_d = tmpdir.join("link")
-        FakeArgs = namedtuple("FakeArgs", ["action", "local", "mode"])
+    def test_status_wrapper_errors(
+        self, action, name, match, caplog, mock_status_wrapper
+    ):
         my_action = mock.Mock()
 
         myargs = FakeArgs((action, my_action), False, "bogusmode")
         with pytest.raises(ValueError, match=match):
-            cli.status_wrapper(name, myargs, data_d, link_d)
+            cli.status_wrapper(name, myargs)
         assert [] == my_action.call_args_list
 
     @mock.patch("cloudinit.cmd.main.atomic_helper.write_json")
     def test_status_wrapper_init_local_writes_fresh_status_info(
         self,
         m_json,
-        tmpdir,
+        mock_status_wrapper,
     ):
         """When running in init-local mode, status_wrapper writes status.json.
 
         Old status and results artifacts are also removed.
         """
-        data_d = tmpdir.join("data")
-        link_d = tmpdir.join("link")
+        data_d = mock_status_wrapper.data_d
+        link_d = mock_status_wrapper.link_d
         # Write old artifacts which will be removed or updated.
         for _dir in data_d, link_d:
             test_helpers.populate_dir(
                 str(_dir), {"status.json": "old", "result.json": "old"}
             )
 
-        FakeArgs = namedtuple("FakeArgs", ["action", "local", "mode"])
-
         def myaction(name, args):
             # Return an error to watch status capture them
             return "SomeDatasource", ["an error"]
 
         myargs = FakeArgs(("ignored_name", myaction), True, "bogusmode")
-        cli.status_wrapper("init", myargs, data_d, link_d)
+        cli.status_wrapper("init", myargs)
         # No errors reported in status
         status_v1 = m_json.call_args_list[1][0][1]["v1"]
         assert status_v1.keys() == {
@@ -109,23 +123,23 @@ class TestCLI:
 
     @mock.patch("cloudinit.cmd.main.atomic_helper.write_json")
     def test_status_wrapper_init_local_honor_cloud_dir(
-        self, m_json, mocker, tmpdir
+        self, m_json, mocker, mock_status_wrapper
     ):
         """When running in init-local mode, status_wrapper honors cloud_dir."""
-        cloud_dir = tmpdir.join("cloud")
+        cloud_dir = mock_status_wrapper.tmpdir.join("cloud")
         paths = helpers.Paths({"cloud_dir": str(cloud_dir)})
-        mocker.patch(M_PATH + "read_cfg_paths", return_value=paths)
-        data_d = cloud_dir.join("data")
-        link_d = tmpdir.join("link")
-
-        FakeArgs = namedtuple("FakeArgs", ["action", "local", "mode"])
+        mocker.patch(
+            "cloudinit.config.schema.read_cfg_paths", return_value=paths
+        )
+        data_d = mock_status_wrapper.data_d
+        link_d = mock_status_wrapper.link_d
 
         def myaction(name, args):
             # Return an error to watch status capture them
             return "SomeDatasource", ["an_error"]
 
         myargs = FakeArgs(("ignored_name", myaction), True, "bogusmode")
-        cli.status_wrapper("init", myargs, link_d=link_d)  # No explicit data_d
+        cli.status_wrapper("init", myargs)  # No explicit data_d
 
         # Access cloud_dir directly
         status_v1 = m_json.call_args_list[1][0][1]["v1"]
@@ -146,9 +160,7 @@ class TestCLI:
 
     def test_no_arguments_shows_error_message(self, capsys):
         exit_code = self._call_main()
-        missing_subcommand_message = (
-            "the following arguments are required: subcommand"
-        )
+        missing_subcommand_message = "a subcommand is required"
         _out, err = capsys.readouterr()
         assert (
             missing_subcommand_message in err
@@ -193,7 +205,7 @@ class TestCLI:
             ),
         ),
     )
-    @mock.patch("cloudinit.cmd.main.setup_basic_logging")
+    @mock.patch("cloudinit.cmd.main.log.setup_basic_logging")
     def test_subcommands_log_to_stderr_via_setup_basic_logging(
         self, setup_basic_logging, subcommand, log_to_stderr, mocks
     ):
@@ -233,9 +245,11 @@ class TestCLI:
             "schema",
         ],
     )
-    @mock.patch("cloudinit.stages.Init._read_cfg", return_value={})
     def test_conditional_subcommands_from_entry_point_sys_argv(
-        self, m_read_cfg, subcommand, capsys, mock_get_user_data_file, tmpdir
+        self,
+        subcommand,
+        capsys,
+        mock_status_wrapper,
     ):
         """Subcommands from entry-point are properly parsed from sys.argv."""
         expected_error = f"usage: cloud-init {subcommand}"
@@ -256,7 +270,7 @@ class TestCLI:
             "status",
         ],
     )
-    def test_subcommand_parser(self, subcommand, mock_get_user_data_file):
+    def test_subcommand_parser(self, subcommand, mock_status_wrapper):
         """cloud-init `subcommand` calls its subparser."""
         # Provide -h param to `subcommand` to avoid having to mock behavior.
         out = io.StringIO()
@@ -300,16 +314,16 @@ class TestCLI:
                 ["all"],
                 [
                     "**Supported distros:** all",
-                    "**Supported distros:** almalinux, alpine, centos, "
-                    "cloudlinux, cos, debian, eurolinux, fedora, freebsd, "
-                    "mariner, miraclelinux, "
-                    "openbsd, openeuler, OpenCloudOS, openmandriva, "
-                    "opensuse, opensuse-microos, opensuse-tumbleweed, "
-                    "opensuse-leap, photon, rhel, rocky, sle_hpc, "
-                    "sle-micro, sles, TencentOS, ubuntu, virtuozzo",
+                    "**Supported distros:** "
+                    "almalinux, alpine, aosc, azurelinux, "
+                    "centos, cloudlinux, cos, debian, eurolinux, fedora, "
+                    "freebsd, mariner, miraclelinux, openbsd, openeuler, "
+                    "OpenCloudOS, openmandriva, opensuse, opensuse-microos, "
+                    "opensuse-tumbleweed, opensuse-leap, photon, rhel, rocky, "
+                    "sle_hpc, sle-micro, sles, TencentOS, ubuntu, virtuozzo",
                     " **resize_rootfs:** ",
                     "(``true``/``false``/``noblock``)",
-                    "runcmd:\n             - [ ls, -l, / ]\n",
+                    "runcmd:\n         - [ls, -l, /]\n",
                 ],
                 False,
                 id="all_spot_check",
@@ -342,7 +356,13 @@ class TestCLI:
     )
     @mock.patch("cloudinit.stages.Init._read_cfg", return_value={})
     def test_wb_schema_subcommand(
-        self, m_read_cfg, args, expected_doc_sections, is_error
+        self,
+        m_read_cfg,
+        args,
+        expected_doc_sections,
+        is_error,
+        mocker,
+        request,
     ):
         """Validate that doc content has correct values."""
 
@@ -356,6 +376,12 @@ class TestCLI:
             if is_error
             else contextlib.redirect_stdout
         )
+        paths = helpers.Paths(
+            {"docs_dir": os.path.join(request.config.rootdir, "doc")}
+        )
+        mocker.patch(
+            "cloudinit.config.schema.read_cfg_paths", return_value=paths
+        )
         with redirecter(out_or_err):
             self._call_main(["cloud-init", "schema", "--docs"] + args)
         out_or_err = out_or_err.getvalue()
@@ -386,3 +412,136 @@ class TestCLI:
         assert "features" == parseargs.action[0]
         assert False is parseargs.debug
         assert False is parseargs.force
+
+
+class TestSignalHandling:
+    @mock.patch("cloudinit.cmd.main.atomic_helper.write_json")
+    def test_status_wrapper_signal_sys_exit(
+        self,
+        m_json,
+        mocker,
+        mock_status_wrapper,
+    ):
+        """make sure that when sys.exit(N) is called, the correct code is
+        returned
+        """
+        for code in [1, 2, 3, 4]:
+            rc = cli.status_wrapper(
+                "init",
+                FakeArgs(
+                    (
+                        None,
+                        # silence pylint false positive
+                        # https://github.com/pylint-dev/pylint/issues/9557
+                        lambda *_: sys.exit(code),  # pylint: disable=W0640
+                    ),
+                    False,
+                    "bogusmode",
+                ),
+            )
+            assert 1 == rc
+
+            # assert that the status shows errors
+            assert (
+                f"sys.exit({code}) called"
+                in m_json.call_args[0][1]["v1"]["init"]["errors"]
+            )
+
+    @mock.patch("cloudinit.cmd.main.atomic_helper.write_json")
+    def test_status_wrapper_no_signal_sys_exit(
+        self,
+        m_json,
+        mock_status_wrapper,
+    ):
+        """if sys.exit(0) is called, make sure that cloud-init doesn't log a
+        warning"""
+        # call status_wrapper() with the required args
+        rc = cli.status_wrapper(
+            "init",
+            FakeArgs(
+                (
+                    None,
+                    lambda *_: sys.exit(0),
+                ),
+                False,
+                "bogusmode",
+            ),
+        )
+        assert 0 == rc
+        assert not m_json.call_args[0][1]["v1"]["init"]["errors"]
+
+    @mock.patch("cloudinit.cmd.main.atomic_helper.write_json")
+    def test_status_wrapper_signal_warnings(
+        self,
+        m_json,
+        mock_status_wrapper,
+    ):
+        """If a stage is started and status.json already has a start time but
+        no end time for that stage, this is an unknown state - make sure that
+        a warning is logged.
+        """
+
+        # Write a status.json to the mocked temporary directory
+        for dir in mock_status_wrapper.data_d, mock_status_wrapper.link_d:
+            test_helpers.populate_dir(
+                str(dir),
+                {
+                    "status.json": json.dumps(
+                        {
+                            "v1": {
+                                "stage": "init",
+                                "datasource": (
+                                    "DataSourceNoCloud "
+                                    "[seed=/var/.../seed/nocloud-net]"
+                                    "[dsmode=net]"
+                                ),
+                                "init": {
+                                    "errors": [],
+                                    "recoverable_errors": {},
+                                    "start": 124.567,
+                                    "finished": None,
+                                },
+                                "init-local": {
+                                    "errors": [],
+                                    "recoverable_errors": {},
+                                    "start": 100.0,
+                                    "finished": 100.00001,
+                                },
+                                "modules-config": {
+                                    "errors": [],
+                                    "recoverable_errors": {},
+                                    "start": None,
+                                    "finished": None,
+                                },
+                                "modules-final": {
+                                    "errors": [],
+                                    "recoverable_errors": {},
+                                    "start": None,
+                                    "finished": None,
+                                },
+                            }
+                        }
+                    )
+                },
+            )
+        # call status_wrapper() with the required args
+        cli.status_wrapper(
+            "init",
+            FakeArgs(
+                (
+                    None,
+                    lambda *_: ("SomeDataSource", []),
+                ),
+                False,
+                "bogusmode",
+            ),
+        )
+
+        # assert that the status shows recoverable errors
+        assert (
+            "Unexpected start time found for Network Stage. "
+            "Was this stage restarted?"
+            in m_json.call_args[0][1]["v1"]["init"]["recoverable_errors"][
+                "WARNING"
+            ]
+        )
