@@ -3,13 +3,16 @@
 import copy
 import os
 from collections import namedtuple
+from logging import getLogger
 from pathlib import Path
+from tempfile import mkdtemp
 from textwrap import dedent
 from uuid import uuid4
 
 import pytest
+import yaml
 
-from cloudinit import atomic_helper, safeyaml, subp, util
+from cloudinit import atomic_helper, subp, util
 from cloudinit.sources import DataSourceIBMCloud as ds_ibm
 from cloudinit.sources import DataSourceOracle as ds_oracle
 from cloudinit.sources import DataSourceSmartOS as ds_smartos
@@ -21,6 +24,8 @@ from tests.unittests.helpers import (
     populate_dir_with_ts,
 )
 
+LOG = getLogger(__name__)
+
 UNAME_MYSYS = "Linux #83-Ubuntu SMP Wed Jan 18 14:10:15 UTC 2017 x86_64"
 UNAME_PPC64EL = (
     "Linux #106-Ubuntu SMP mon Jun 26 17:53:54 UTC 2017 "
@@ -62,6 +67,146 @@ BLKID_UEFI_UBUNTU = [
 ]
 
 
+DEFAULT_CLOUD_CONFIG = """\
+# The top level settings are used as module
+# and base configuration.
+# A set of users which may be applied and/or used by various modules
+# when a 'default' entry is found it will reference the 'default_user'
+# from the distro configuration specified below
+users:
+   - default
+
+# If this is set, 'root' will not be able to ssh in and they
+# will get a message to login instead as the default $user
+disable_root: true
+
+# This will cause the set+update hostname module to not operate (if true)
+preserve_hostname: false
+
+# If you use datasource_list array, keep array items in a single line.
+# If you use multi line array, ds-identify script won't read array items.
+# Example datasource config
+# datasource:
+#    Ec2:
+#      metadata_urls: [ 'blah.com' ]
+#      timeout: 5 # (defaults to 50 seconds)
+#      max_wait: 10 # (defaults to 120 seconds)
+
+# The modules that run in the 'init' stage
+cloud_init_modules:
+ - migrator
+ - seed_random
+ - bootcmd
+ - write-files
+ - growpart
+ - resizefs
+ - disk_setup
+ - mounts
+ - set_hostname
+ - update_hostname
+ - update_etc_hosts
+ - ca-certs
+ - rsyslog
+ - users-groups
+ - ssh
+
+# The modules that run in the 'config' stage
+cloud_config_modules:
+ - wireguard
+ - snap
+ - ubuntu_autoinstall
+ - ssh-import-id
+ - keyboard
+ - locale
+ - set-passwords
+ - grub-dpkg
+ - apt-pipelining
+ - apt-configure
+ - ubuntu-advantage
+ - ntp
+ - timezone
+ - disable-ec2-metadata
+ - runcmd
+ - byobu
+
+# The modules that run in the 'final' stage
+cloud_final_modules:
+ - package-update-upgrade-install
+ - fan
+ - landscape
+ - lxd
+ - ubuntu-drivers
+ - write-files-deferred
+ - puppet
+ - chef
+ - ansible
+ - mcollective
+ - salt-minion
+ - reset_rmc
+ - refresh_rmc_and_interface
+ - rightscale_userdata
+ - scripts-vendor
+ - scripts-per-once
+ - scripts-per-boot
+ - scripts-per-instance
+ - scripts-user
+ - ssh-authkey-fingerprints
+ - keys-to-console
+ - install-hotplug
+ - phone-home
+ - final-message
+ - power-state-change
+
+# System and/or distro specific settings
+# (not accessible to handlers/transforms)
+system_info:
+   # This will affect which distro class gets used
+   distro: ubuntu
+   # Default user name + that default users groups (if added/used)
+   default_user:
+     name: ubuntu
+     lock_passwd: True
+     gecos: Ubuntu
+     groups: [adm, audio, cdrom, floppy, lxd, netdev, plugdev, sudo, video]
+     sudo: ["ALL=(ALL) NOPASSWD:ALL"]
+     shell: /bin/bash
+   network:
+     renderers: ['netplan', 'eni', 'sysconfig']
+     activators: ['netplan', 'eni', 'network-manager', 'networkd']
+   # Automatically discover the best ntp_client
+   ntp_client: auto
+   # Other config here will be given to the distro class and/or path classes
+   paths:
+      cloud_dir: /var/lib/cloud/
+      templates_dir: /etc/cloud/templates/
+   package_mirrors:
+     - arches: [i386, amd64]
+       failsafe:
+         primary: http://archive.ubuntu.com/ubuntu
+         security: http://security.ubuntu.com/ubuntu
+       search:
+         primary:
+           - http://%(ec2_region)s.ec2.archive.ubuntu.com/ubuntu/
+           - http://%(availability_zone)s.clouds.archive.ubuntu.com/ubuntu/
+           - http://%(region)s.clouds.archive.ubuntu.com/ubuntu/
+         security: []
+     - arches: [arm64, armel, armhf]
+       failsafe:
+         primary: http://ports.ubuntu.com/ubuntu-ports
+         security: http://ports.ubuntu.com/ubuntu-ports
+       search:
+         primary:
+           - http://%(ec2_region)s.ec2.ports.ubuntu.com/ubuntu-ports/
+           - http://%(availability_zone)s.clouds.ports.ubuntu.com/ubuntu-ports/
+           - http://%(region)s.clouds.ports.ubuntu.com/ubuntu-ports/
+         security: []
+     - arches: [default]
+       failsafe:
+         primary: http://ports.ubuntu.com/ubuntu-ports
+         security: http://ports.ubuntu.com/ubuntu-ports
+   ssh_svcname: ssh
+"""
+
 POLICY_FOUND_ONLY = "search,found=all,maybe=none,notfound=disabled"
 POLICY_FOUND_OR_MAYBE = "search,found=all,maybe=all,notfound=disabled"
 DI_DEFAULT_POLICY = "search,found=all,maybe=all,notfound=disabled"
@@ -99,16 +244,22 @@ MOCK_VIRT_IS_CONTAINER_OTHER = {
     "RET": "container-other",
     "ret": 0,
 }
+IS_CONTAINER_OTHER_ENV = {"SYSTEMD_VIRTUALIZATION": "vm:kvm"}
 MOCK_NOT_LXD_DATASOURCE = {"name": "dscheck_LXD", "ret": 1}
 MOCK_VIRT_IS_KVM = {"name": "detect_virt", "RET": "kvm", "ret": 0}
+KVM_ENV = {"SYSTEMD_VIRTUALIZATION": "vm:kvm"}
 # qemu support for LXD is only for host systems > 5.10 kernel as lxd
 # passed `hv_passthrough` which causes systemd < v.251 to misinterpret CPU
 # as "qemu" instead of "kvm"
 MOCK_VIRT_IS_KVM_QEMU = {"name": "detect_virt", "RET": "qemu", "ret": 0}
+IS_KVM_QEMU_ENV = {"SYSTEMD_VIRTUALIZATION": "vm:qemu"}
 MOCK_VIRT_IS_VMWARE = {"name": "detect_virt", "RET": "vmware", "ret": 0}
+IS_VMWARE_ENV = {"SYSTEMD_VIRTUALIZATION": "vm:vmware"}
 # currenty' SmartOS hypervisor "bhyve" is unknown by systemd-detect-virt.
 MOCK_VIRT_IS_VM_OTHER = {"name": "detect_virt", "RET": "vm-other", "ret": 0}
+IS_VM_OTHER = {"SYSTEMD_VIRTUALIZATION": "vm:vm-other"}
 MOCK_VIRT_IS_XEN = {"name": "detect_virt", "RET": "xen", "ret": 0}
+IS_XEN_ENV = {"SYSTEMD_VIRTUALIZATION": "vm:xen"}
 MOCK_VIRT_IS_WSL = {"name": "detect_virt", "RET": "wsl", "ret": 0}
 MOCK_UNAME_IS_PPC64 = {"name": "uname", "out": UNAME_PPC64EL, "ret": 0}
 MOCK_UNAME_IS_FREEBSD = {"name": "uname", "out": UNAME_FREEBSD, "ret": 0}
@@ -153,6 +304,9 @@ class DsIdentifyBase(CiTestCase):
     dsid_path = cloud_init_project_dir("tools/ds-identify")
     allowed_subp = ["sh"]
 
+    # set to true to write out the mocked ds-identify for inspection
+    debug_mode = True
+
     def call(
         self,
         rootd=None,
@@ -164,6 +318,7 @@ class DsIdentifyBase(CiTestCase):
         policy_dmi=DI_DEFAULT_POLICY,
         policy_no_dmi=DI_DEFAULT_POLICY_NO_DMI,
         ec2_strict_id=DI_EC2_STRICT_ID_DEFAULT,
+        env_vars=None,
     ):
         if args is None:
             args = []
@@ -173,6 +328,10 @@ class DsIdentifyBase(CiTestCase):
         if files is None:
             files = {}
 
+        cloudcfg = "etc/cloud/cloud.cfg"
+        if cloudcfg not in files:
+            files[cloudcfg] = DEFAULT_CLOUD_CONFIG
+
         if rootd is None:
             rootd = self.tmp_dir()
 
@@ -251,12 +410,26 @@ class DsIdentifyBase(CiTestCase):
 
         endlines = [func + " " + " ".join(['"%s"' % s for s in args])]
 
+        mocked_ds_identify = "\n".join(head + mocklines + endlines) + "\n"
         with open(wrap, "w") as fp:
-            fp.write("\n".join(head + mocklines + endlines) + "\n")
+            fp.write(mocked_ds_identify)
+
+        # debug_mode force this test to write the mocked ds-identify script to
+        # a file for inspection
+        if self.debug_mode:
+            tempdir = mkdtemp()
+            dir = f"{tempdir}/ds-identify"
+            LOG.debug("Writing mocked ds-identify to %s for debugging.", dir)
+            with open(dir, "w") as fp:
+                fp.write(mocked_ds_identify)
 
         rc = 0
         try:
-            out, err = subp.subp(["sh", "-c", ". %s" % wrap], capture=True)
+            out, err = subp.subp(
+                ["sh", "-c", ". %s" % wrap],
+                update_env=env_vars if env_vars else {},
+                capture=True,
+            )
         except subp.ProcessExecutionError as e:
             rc = e.exit_code
             out = e.stdout
@@ -267,7 +440,7 @@ class DsIdentifyBase(CiTestCase):
         if os.path.exists(cfg_out):
             contents = util.load_text_file(cfg_out)
             try:
-                cfg = safeyaml.load(contents)
+                cfg = yaml.safe_load(contents)
             except Exception as e:
                 cfg = {"_INVALID_YAML": contents, "_EXCEPTION": str(e)}
 
@@ -281,6 +454,7 @@ class DsIdentifyBase(CiTestCase):
             "mocks",
             "func",
             "args",
+            "env_vars",
             "policy_dmi",
             "policy_no_dmi",
             "files",
@@ -294,8 +468,9 @@ class DsIdentifyBase(CiTestCase):
 
     def _test_ds_found(self, name):
         data = copy.deepcopy(VALID_CFG[name])
+
         return self._check_via_dict(
-            data, RC_FOUND, dslist=[data.get("ds"), DS_NONE]
+            data, RC_FOUND, dslist=[data.pop("ds"), DS_NONE]
         )
 
     def _test_ds_not_found(self, name):
@@ -308,7 +483,7 @@ class DsIdentifyBase(CiTestCase):
         try:
             self.assertEqual(rc, ret.rc)
             if dslist is not None:
-                self.assertEqual(dslist, ret.cfg["datasource_list"])
+                self.assertEqual(dslist, ret.cfg.get("datasource_list"))
             good = True
         finally:
             if not good:
@@ -378,6 +553,7 @@ class TestDsIdentify(DsIdentifyBase):
         config = "LXD-kvm-not-MAAS-2"
         self._test_ds_found(config)
 
+    @pytest.mark.xfail(reason="GH-4796")
     def test_maas_not_detected_3(self):
         """Don't incorrectly identify maas
 
@@ -395,6 +571,21 @@ class TestDsIdentify(DsIdentifyBase):
         config = "LXD-kvm-not-MAAS-3"
         self._test_ds_found(config)
 
+    def test_flow_sequence_control(self):
+        """ensure that an invalid key in the flow_sequence tests produces no
+        datasource list match
+
+        control test: this test serves as a control test for test_flow_sequence
+        """
+        data = copy.deepcopy(VALID_CFG["flow_sequence-control"])
+        self._check_via_dict(data, RC_NOT_FOUND)
+
+    def test_flow_sequence(self):
+        """correctly identify flow sequences"""
+        for i in range(1, 10):
+            data = copy.deepcopy(VALID_CFG[f"flow_sequence-{i}"])
+            self._check_via_dict(data, RC_FOUND, dslist=[data.get("ds")])
+
     def test_azure_invalid_configuration(self):
         """Don't detect incorrect config when invalid datasource_list provided
 
@@ -419,6 +610,20 @@ class TestDsIdentify(DsIdentifyBase):
         """EC2: hvm instances use dmi serial and uuid starting with 'ec2'."""
         self._test_ds_found("Ec2-hvm")
 
+    def test_aws_ec2_hvm_env(self):
+        """EC2: hvm instances use dmi serial and uuid starting with 'ec2'
+
+        test using SYSTEMD_VIRTUALIZATION, not systemd-detect-virt
+        """
+        self._test_ds_found("Ec2-hvm-env")
+
+    def test_aws_ec2_hvm_endian(self):
+        """EC2: hvm instances use system-uuid and may have swapped endianness
+
+        test using SYSTEMD_VIRTUALIZATION, not systemd-detect-virt
+        """
+        self._test_ds_found("Ec2-hvm-swap-endianness")
+
     def test_aws_ec2_xen(self):
         """EC2: sys/hypervisor/uuid starts with ec2."""
         self._test_ds_found("Ec2-xen")
@@ -439,6 +644,13 @@ class TestDsIdentify(DsIdentifyBase):
         """GCE identifies itself with product_name."""
         self._test_ds_found("GCE")
 
+    def test_gce_by_product_name_env(self):
+        """GCE identifies itself with product_name.
+
+        Uses SYSTEMD_VIRTUALIZATION
+        """
+        self._test_ds_found("GCE_ENV")
+
     def test_gce_by_serial(self):
         """Older gce compute instances must be identified by serial."""
         self._test_ds_found("GCE-serial")
@@ -461,6 +673,20 @@ class TestDsIdentify(DsIdentifyBase):
         """
         self._test_ds_found("LXD-kvm-qemu-kernel-gt-5.10")
 
+    def test_lxd_kvm_jammy_env(self):
+        """LXD KVM on host systems with a kernel > 5.10 need to match "qemu".
+        LXD provides `hv_passthrough` when launching kvm instances when host
+        kernel is > 5.10. This results in systemd being unable to detect the
+        virtualized CPUID="Linux KVM Hv" as type "kvm" and results in
+        systemd-detect-virt returning "qemu" in this case.
+
+        Assert ds-identify can match systemd-detect-virt="qemu" and
+        /sys/class/dmi/id/board_name = LXD.
+        Once systemd 251 is available on a target distro, the virtualized
+        CPUID will be represented properly as "kvm"
+        """
+        self._test_ds_found("LXD-kvm-qemu-kernel-gt-5.10-env")
+
     def test_lxd_containers(self):
         """LXD containers will have /dev/lxd/socket at generator time."""
         self._test_ds_found("LXD")
@@ -509,6 +735,13 @@ class TestDsIdentify(DsIdentifyBase):
             ret.cfg.get("datasource_list"), ["ConfigDrive", "None"]
         )
 
+    @pytest.mark.xfail(
+        reason=("not supported: yaml parser implemented in POSIX shell")
+    )
+    def test_multiline_yaml(self):
+        """Multi-line yaml is unsupported"""
+        self._test_ds_found("LXD-kvm-not-azure")
+
     def test_ibmcloud_template_userdata_in_provisioning(self):
         """Template provisioned with user-data during provisioning stage.
 
@@ -634,6 +867,32 @@ class TestDsIdentify(DsIdentifyBase):
         mydata["files"][cfgpath] = 'datasource_list: ["Ec2", "None"]\n'
         self._check_via_dict(mydata, rc=RC_FOUND, dslist=["Ec2", DS_NONE])
 
+    def test_nocloud_seedfrom(self):
+        """Check seedfrom system config detects nocloud.
+
+        Verify that a cloud.cfg.d/ that contains more than two datasources in
+        its datasource_list will positively identify nocloud when a
+        datasource.NoCloud.seedfrom value exists
+        """
+        self._check_via_dict(
+            copy.deepcopy(VALID_CFG["NoCloud-seedfrom"]),
+            rc=RC_FOUND,
+            dslist=["NoCloud", DS_NONE],
+        )
+
+    def test_nocloud_userdata_and_metadata(self):
+        """Check seedfrom system config detects nocloud.
+
+        Verify that a cloud.cfg.d/ that contains more than two datasources in
+        its datasource_list will positively identify nocloud when both
+        datasource.NoCloud.{user-data,meta-data} value exists
+        """
+        self._check_via_dict(
+            copy.deepcopy(VALID_CFG["NoCloud-user-data-meta-data"]),
+            rc=RC_FOUND,
+            dslist=["NoCloud", DS_NONE],
+        )
+
     def test_aliyun_identified(self):
         """Test that Aliyun cloud is identified by product id."""
         self._test_ds_found("AliYun")
@@ -661,6 +920,10 @@ class TestDsIdentify(DsIdentifyBase):
         """SAP Converged Cloud identification"""
         self._test_ds_found("OpenStack-SAPCCloud")
 
+    def test_openstack_sap_ccloud_env(self):
+        """SAP Converged Cloud identification"""
+        self._test_ds_found("OpenStack-SAPCCloud-env")
+
     def test_openstack_huawei_cloud(self):
         """Open Huawei Cloud identification."""
         self._test_ds_found("OpenStack-HuaweiCloud")
@@ -677,7 +940,7 @@ class TestDsIdentify(DsIdentifyBase):
         """On non-Intel, openstack without dmi info is maybe.
 
         nova does not identify itself on platforms other than intel.
-           https://bugs.launchpad.net/cloud-init/+bugs?field.tag=dsid-nova"""
+        https://bugs.launchpad.net/cloud-init/+bugs?field.tag=dsid-nova"""
 
         data = copy.deepcopy(VALID_CFG["OpenStack"])
         del data["files"][P_PRODUCT_NAME]
@@ -828,12 +1091,22 @@ class TestDsIdentify(DsIdentifyBase):
         """SmartOS cloud identified on lxbrand container."""
         self._test_ds_found("SmartOS-lxbrand")
 
+    def test_smartos_lxbrand_env(self):
+        """SmartOS cloud identified on lxbrand container."""
+        self._test_ds_found("SmartOS-lxbrand-env")
+
     def test_smartos_lxbrand_requires_socket(self):
         """SmartOS cloud should not be identified if no socket file."""
         mycfg = copy.deepcopy(VALID_CFG["SmartOS-lxbrand"])
         del mycfg["files"][ds_smartos.METADATA_SOCKFILE]
         self._check_via_dict(mycfg, rc=RC_NOT_FOUND, policy_dmi="disabled")
 
+    def test_smartos_lxbrand_requires_socket_env(self):
+        """SmartOS cloud should not be identified if no socket file."""
+        mycfg = copy.deepcopy(VALID_CFG["SmartOS-lxbrand-env"])
+        del mycfg["files"][ds_smartos.METADATA_SOCKFILE]
+        self._check_via_dict(mycfg, rc=RC_NOT_FOUND, policy_dmi="disabled")
+
     def test_path_env_gets_set_from_main(self):
         """PATH environment should always have some tokens when main is run.
 
@@ -1111,14 +1384,29 @@ class TestWSL(DsIdentifyBase):
         """Negative test by lack of host filesystem mount points."""
         self._test_ds_not_found("WSL-no-host-mounts")
 
-    def test_no_cloudinitdir(self):
-        """Negative test by not finding %USERPROFILE%/.cloud-init."""
+    def test_no_userprofile(self):
+        """Negative test by failing to read the %USERPROFILE% environment
+        variable.
+        """
         data = copy.deepcopy(VALID_CFG["WSL-supported"])
         data["mocks"].append(
             {
-                "name": "WSL_cloudinit_dir_in",
-                "ret": 1,
-                "RET": "",
+                "name": "WSL_run_cmd",
+                "ret": 0,
+                "RET": "\r\n",
+            },
+        )
+        return self._check_via_dict(data, RC_NOT_FOUND)
+
+    def test_no_cloudinitdir_in_userprofile(self):
+        """Negative test by not finding %USERPROFILE%/.cloud-init."""
+        data = copy.deepcopy(VALID_CFG["WSL-supported"])
+        userprofile = self.tmp_dir()
+        data["mocks"].append(
+            {
+                "name": "WSL_profile_dir",
+                "ret": 0,
+                "RET": userprofile,
             },
         )
         return self._check_via_dict(data, RC_NOT_FOUND)
@@ -1126,27 +1414,58 @@ class TestWSL(DsIdentifyBase):
     def test_empty_cloudinitdir(self):
         """Negative test by lack of host filesystem mount points."""
         data = copy.deepcopy(VALID_CFG["WSL-supported"])
-        cloudinitdir = self.tmp_dir()
+        userprofile = self.tmp_dir()
         data["mocks"].append(
             {
-                "name": "WSL_cloudinit_dir_in",
+                "name": "WSL_profile_dir",
                 "ret": 0,
-                "RET": cloudinitdir,
+                "RET": userprofile,
             },
         )
+        cloudinitdir = os.path.join(userprofile, ".cloud-init")
+        os.mkdir(cloudinitdir)
         return self._check_via_dict(data, RC_NOT_FOUND)
 
-    def test_found_via_userdata_version_codename(self):
-        """WLS datasource detected by VERSION_CODENAME when no VERSION_ID"""
+    def test_found_fail_due_instance_name_parsing(self):
+        """WSL datasource detection fail due parsing error even though the file
+        exists.
+        """
         data = copy.deepcopy(VALID_CFG["WSL-supported-debian"])
-        cloudinitdir = self.tmp_dir()
+        userprofile = self.tmp_dir()
         data["mocks"].append(
             {
-                "name": "WSL_cloudinit_dir_in",
+                "name": "WSL_profile_dir",
                 "ret": 0,
-                "RET": cloudinitdir,
+                "RET": userprofile,
             },
         )
+
+        # Forcing WSL_linux2win_path to return a path we'll fail to parse
+        # (missing one / in the begining of the path).
+        for i, m in enumerate(data["mocks"]):
+            if m["name"] == "WSL_linux2win_path":
+                data["mocks"][i]["RET"] = "/wsl.localhost/cant-findme"
+
+        cloudinitdir = os.path.join(userprofile, ".cloud-init")
+        os.mkdir(cloudinitdir)
+        filename = os.path.join(cloudinitdir, "cant-findme.user-data")
+        Path(filename).touch()
+        self._check_via_dict(data, RC_NOT_FOUND)
+        Path(filename).unlink()
+
+    def test_found_via_userdata_version_codename(self):
+        """WSL datasource detected by VERSION_CODENAME when no VERSION_ID"""
+        data = copy.deepcopy(VALID_CFG["WSL-supported-debian"])
+        userprofile = self.tmp_dir()
+        data["mocks"].append(
+            {
+                "name": "WSL_profile_dir",
+                "ret": 0,
+                "RET": userprofile,
+            },
+        )
+        cloudinitdir = os.path.join(userprofile, ".cloud-init")
+        os.mkdir(cloudinitdir)
         filename = os.path.join(cloudinitdir, "debian-trixie.user-data")
         Path(filename).touch()
         self._check_via_dict(data, RC_FOUND, dslist=[data.get("ds"), DS_NONE])
@@ -1157,15 +1476,23 @@ class TestWSL(DsIdentifyBase):
         WSL datasource is found on applicable userdata files in cloudinitdir.
         """
         data = copy.deepcopy(VALID_CFG["WSL-supported"])
-        cloudinitdir = self.tmp_dir()
+        userprofile = self.tmp_dir()
         data["mocks"].append(
             {
-                "name": "WSL_cloudinit_dir_in",
+                "name": "WSL_profile_dir",
                 "ret": 0,
-                "RET": cloudinitdir,
+                "RET": userprofile,
             },
         )
+        cloudinitdir = os.path.join(userprofile, ".cloud-init")
+        os.mkdir(cloudinitdir)
+        up4wcloudinitdir = os.path.join(userprofile, ".ubuntupro/.cloud-init")
+        os.makedirs(up4wcloudinitdir, exist_ok=True)
         userdata_files = [
+            os.path.join(
+                up4wcloudinitdir, MOCK_WSL_INSTANCE_DATA["name"] + ".user-data"
+            ),
+            os.path.join(up4wcloudinitdir, "agent.yaml"),
             os.path.join(
                 cloudinitdir, MOCK_WSL_INSTANCE_DATA["name"] + ".user-data"
             ),
@@ -1298,6 +1625,21 @@ VALID_CFG = {
             P_PRODUCT_UUID: "EC23AEF5-54BE-4843-8D24-8C819F88453E\n",
         },
     },
+    "Ec2-hvm-swap-endianness": {
+        "ds": "Ec2",
+        "mocks": [{"name": "detect_virt", "RET": "kvm", "ret": 0}],
+        "files": {
+            P_PRODUCT_UUID: "AB232AEC-54BE-4843-8D24-8C819F88453E\n",
+        },
+    },
+    "Ec2-hvm-env": {
+        "ds": "Ec2",
+        "mocks": [{"name": "detect_virt_env", "RET": "vm:kvm", "ret": 0}],
+        "files": {
+            P_PRODUCT_SERIAL: "ec23aef5-54be-4843-8d24-8c819f88453e\n",
+            P_PRODUCT_UUID: "EC23AEF5-54BE-4843-8D24-8C819F88453E\n",
+        },
+    },
     "Ec2-xen": {
         "ds": "Ec2",
         "mocks": [MOCK_VIRT_IS_XEN],
@@ -1318,6 +1660,12 @@ VALID_CFG = {
         "files": {P_PRODUCT_NAME: "Google Compute Engine\n"},
         "mocks": [MOCK_VIRT_IS_KVM],
     },
+    "GCE_ENV": {
+        "ds": "GCE",
+        "files": {P_PRODUCT_NAME: "Google Compute Engine\n"},
+        "env_vars": KVM_ENV,
+        "no_mocks": ["detect_virt"],
+    },
     "GCE-serial": {
         "ds": "GCE",
         "files": {P_PRODUCT_SERIAL: "GoogleCloud-8f2e88f\n"},
@@ -1356,7 +1704,137 @@ VALID_CFG = {
         "ds": "LXD",
         "files": {
             P_BOARD_NAME: "LXD\n",
-            "etc/cloud/cloud.cfg.d/92-broken-maas.cfg": ("MAAS: None"),
+            "etc/cloud/cloud.cfg.d/92-broken-maas.cfg": ("MAAS: None\n"),
+        },
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+    },
+    "flow_sequence-control": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {
+            "etc/cloud/cloud.cfg": dedent(
+                """\
+                "datasource-list":  [ None    ]   \n
+                """
+            )
+        },
+    },
+    # no quotes, whitespace between all chars and at the end of line
+    "flow_sequence-1": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {
+            "etc/cloud/cloud.cfg": dedent(
+                """\
+                datasource_list :  [ None    ]   \n
+                """
+            )
+        },
+    },
+    # double quotes
+    "flow_sequence-2": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {
+            "etc/cloud/cloud.cfg": dedent(
+                """\
+                "datasource_list": [None]
+                """
+            )
+        },
+    },
+    # single quotes
+    "flow_sequence-3": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {
+            "etc/cloud/cloud.cfg": dedent(
+                """\
+                'datasource_list': [None]
+                """
+            )
+        },
+    },
+    # no newlines
+    "flow_sequence-4": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {
+            "etc/cloud/cloud.cfg": dedent("datasource_list:  [ None     ]")
+        },
+    },
+    # double quoted key, single quoted list member
+    "flow_sequence-5": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {
+            "etc/cloud/cloud.cfg": dedent(
+                "\"datasource_list\": [    'None' ]  "
+            )
+        },
+    },
+    # single quotes, whitespace before colon
+    "flow_sequence-6": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {
+            "etc/cloud/cloud.cfg": dedent("'datasource_list' : [    None  ]  ")
+        },
+    },
+    "flow_sequence-7": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {
+            "etc/cloud/cloud.cfg": dedent(
+                '"datasource_list"     : [    None  ]  '
+            )
+        },
+    },
+    # tabs as part of whitespace between all chars
+    "flow_sequence-8": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {
+            "etc/cloud/cloud.cfg": dedent(
+                '"datasource_list"   \t\t  : \t\t[\t   \tNone \t \t ] \t\t '
+            )
+        },
+    },
+    # no quotes, no whitespace
+    "flow_sequence-9": {
+        "ds": "None",
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
+        "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
+        "files": {"etc/cloud/cloud.cfg": dedent("datasource_list: [None]")},
+    },
+    "LXD-kvm-not-azure": {
+        "ds": "Azure",
+        "files": {
+            P_BOARD_NAME: "LXD\n",
+            "etc/cloud/cloud.cfg.d/92-broken-azure.cfg": (
+                "datasource_list:\n - Azure"
+            ),
         },
         # /dev/lxd/sock does not exist and KVM virt-type
         "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM],
@@ -1369,6 +1847,28 @@ VALID_CFG = {
         "mocks": [{"name": "is_socket_file", "ret": 1}, MOCK_VIRT_IS_KVM_QEMU],
         "no_mocks": ["dscheck_LXD"],  # Don't default mock dscheck_LXD
     },
+    # LXD host > 5.10 kvm launch virt==qemu
+    "LXD-kvm-qemu-kernel-gt-5.10-env": {
+        "ds": "LXD",
+        "files": {
+            P_BOARD_NAME: "LXD\n",
+            # this test is systemd-specific, but may run on non-systemd systems
+            # ensure that /run/systemd/ exists, such that this test will take
+            # the systemd branch on those systems as well
+            #
+            # https://github.com/canonical/cloud-init/issues/5095
+            "/run/systemd/somefile": "",
+        },
+        # /dev/lxd/sock does not exist and KVM virt-type
+        "mocks": [
+            {"name": "is_socket_file", "ret": 1},
+        ],
+        "env_vars": IS_KVM_QEMU_ENV,
+        "no_mocks": [
+            "dscheck_LXD",
+            "detect_virt",
+        ],  # Don't default mock dscheck_LXD
+    },
     "LXD": {
         "ds": "LXD",
         # /dev/lxd/sock exists
@@ -1404,9 +1904,9 @@ VALID_CFG = {
             # Also include a datasource list of more than just
             # [NoCloud, None], because that would automatically select
             # NoCloud without checking
-            "/etc/cloud/cloud.cfg": dedent(
+            "etc/cloud/cloud.cfg": dedent(
                 """\
-                datasource_list: [ Azure, Openstack, NoCloud, None ]
+                datasource_list: [ Azure, OpenStack, NoCloud, None ]
                 datasource:
                   NoCloud:
                     user-data: |
@@ -1490,6 +1990,41 @@ VALID_CFG = {
             os.path.join(P_SEED_DIR, "nocloud", "meta-data"): "md\n",
         },
     },
+    "NoCloud-seedfrom": {
+        "ds": "NoCloud",
+        "files": {
+            # Also include a datasource list of more than just
+            # [NoCloud, None], because that would automatically select
+            # NoCloud without checking
+            "etc/cloud/cloud.cfg.d/test.cfg": dedent(
+                """\
+                datasource_list: [ Azure, OpenStack, NoCloud, None ]
+                datasource:
+                  NoCloud:
+                    seedfrom: http://0.0.0.0/test
+                """
+            )
+        },
+    },
+    "NoCloud-user-data-meta-data": {
+        "ds": "NoCloud",
+        "files": {
+            # Also include a datasource list of more than just
+            # [NoCloud, None], because that would automatically select
+            # NoCloud without checking
+            "etc/cloud/cloud.cfg.d/test.cfg": dedent(
+                """\
+                datasource_list: [ Azure, OpenStack, NoCloud, None ]
+                datasource:
+                  NoCloud:
+                    meta-data: ""
+                    user-data: |
+                      #cloud-config
+                      <some-config>
+            """
+            )
+        },
+    },
     "NoCloud-seed-ubuntu-core": {
         "ds": "NoCloud",
         "files": {
@@ -1520,6 +2055,13 @@ VALID_CFG = {
         "files": {P_CHASSIS_ASSET_TAG: "SAP CCloud VM\n"},
         "mocks": [MOCK_VIRT_IS_VMWARE],
     },
+    "OpenStack-SAPCCloud-env": {
+        # SAP CCloud hosts use OpenStack on VMware
+        "ds": "OpenStack",
+        "files": {P_CHASSIS_ASSET_TAG: "SAP CCloud VM\n"},
+        "env_vars": IS_VMWARE_ENV,
+        "no_mocks": ["detect_virt"],
+    },
     "OpenStack-HuaweiCloud": {
         # Huawei Cloud hosts use OpenStack
         "ds": "OpenStack",
@@ -1878,6 +2420,20 @@ VALID_CFG = {
         ],
         "files": {ds_smartos.METADATA_SOCKFILE: "would be a socket\n"},
     },
+    "SmartOS-lxbrand-env": {
+        "ds": "SmartOS",
+        "mocks": [
+            {
+                "name": "uname",
+                "ret": 0,
+                "out": ("Linux BrandZ virtual linux x86_64"),
+            },
+            {"name": "blkid", "ret": 2, "out": ""},
+        ],
+        "no_mocks": ["detect_virt"],
+        "env_vars": IS_CONTAINER_OTHER_ENV,
+        "files": {ds_smartos.METADATA_SOCKFILE: "would be a socket\n"},
+    },
     "Ec2-ZStack": {
         "ds": "Ec2",
         "files": {P_CHASSIS_ASSET_TAG: "123456.zstack.io\n"},
@@ -2241,9 +2797,9 @@ VALID_CFG = {
             MOCK_VIRT_IS_WSL,
             MOCK_UNAME_IS_WSL,
             {
-                "name": "WSL_instance_name",
+                "name": "WSL_path",
                 "ret": 0,
-                "RET": MOCK_WSL_INSTANCE_DATA["name"],
+                "RET": "//wsl.localhost/%s/" % MOCK_WSL_INSTANCE_DATA["name"],
             },
         ],
         "files": {
@@ -2264,9 +2820,9 @@ VALID_CFG = {
             MOCK_VIRT_IS_WSL,
             MOCK_UNAME_IS_WSL,
             {
-                "name": "WSL_instance_name",
+                "name": "WSL_path",
                 "ret": 0,
-                "RET": MOCK_WSL_INSTANCE_DATA["name"],
+                "RET": "//wsl.localhost/%s/" % MOCK_WSL_INSTANCE_DATA["name"],
             },
         ],
         "files": {
